# GNU General Public License (GPL)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#


#CMF
from Products.CMFCore.utils import getToolByName
from Products.CMFFormController.FormAction import FormActionKey
from Products.CMFDynamicViewFTI.migrate import migrateFTIs

#Archetypes
from Products.Archetypes.Extensions.utils import installTypes
from Products.Archetypes.Extensions.utils import install_subskin
from Products.Archetypes.public import listTypes

#SignupSheet
from Products.SignupSheet.config import *

from StringIO import StringIO
from zExceptions import NotFound, BadRequest
from Products.ExternalMethod.ExternalMethod import ExternalMethod

#The following is modified from the Poi Products

def addFormControllerAction(self, out, controller, template, status, 
                                contentType, button, actionType, action):
    """Add the given action to the portalFormController"""
    controller.addFormAction(template, status, contentType,
                                button, actionType, action)
    print >> out, "Added action %s to %s" % (action, template)



def install_dependencies(self):
    qi=getToolByName(self, 'portal_quickinstaller', None)
    if not qi.isProductInstalled('ATSchemaEditorNG'):
        qi.installProduct('ATSchemaEditorNG',locked=0)
        return 'ATSchemaEditorNG \n'






def install(self):
    """Install SignupSheet: Install content types, skin layer, install the
    stylesheet, set up global properties, enable the portal factory and
    set up form controller actions for the widget actions
    """

    out = StringIO()

    print >> out, "Installing SignupSheet"

    # Install types
    classes = listTypes(PROJECTNAME)
    installTypes(self, out,
                 classes,
                 PROJECTNAME)
    print >> out, "Installed types"

    # Install skin
    install_subskin(self, out, product_globals)
    print >> out, "Installed skin"

    # Migrate FTI, to make sure we get the necessary infrastructure for the
    # 'display' menu to work.
    migrated = migrateFTIs(self, product=PROJECTNAME)
    print >>out, "Switched to DynamicViewFTI: %s" % ', '.join(migrated)

    # Install stylesheet
    portal_css = getToolByName(self, 'portal_css')
    portal_css.manage_addStylesheet(id = 'SignupSheet.css',
                                    expression = 'python:object.getTypeInfo().getId() == "SignupSheet"',
                                    media = 'all',
                                    title = 'SignupSheet styles',
                                    enabled = True)
                                    
    # try to call a workflow install method
    # in 'InstallWorkflows.py' method 'installWorkflows'
    try:
        installWorkflows = ExternalMethod('temp','temp',PROJECTNAME+'.InstallWorkflows', 'installWorkflows').__of__(self)
    except NotFound:
        installWorkflows = None

    if installWorkflows:
        print >>out,'Workflow Install:'
        res = installWorkflows(self,out)
        print >>out,res or 'no output'
    else:
        print >>out,'no workflow install'

    #bind classes to workflows
    wft = getToolByName(self,'portal_workflow')
                        
                                    

    # Enable portal_factory
    # enable portal_factory for given types
    factory_tool = getToolByName(self,'portal_factory')
    factory_types=[
        "SignupSheet",
        "Registrant",
        ] + factory_tool.getFactoryTypes().keys()
    factory_tool.manage_setPortalFactoryTypes(listOfTypeIds=factory_types)
    print >> out, "Added SignupSheet and Registrant to portal_factory"

    # Enable portal_factory
    factory = getToolByName(self, 'portal_factory')
    types = factory.getFactoryTypes().keys()
    if 'SignupSheet' not in types:
        types.append('SignupSheet')
        factory.manage_setPortalFactoryTypes(listOfTypeIds = types)

    print >> out, "Added SignupSheet to portal_factory"







    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')
    navtreeProperties = getattr(propsTool, 'navtree_properties')

    # Remove from use_folder_tabs
    useFolderTabs = list(siteProperties.getProperty('use_folder_tabs'))
    if 'SignupSheet' in useFolderTabs:
        useFolderTabs.remove('SignupSheet')
    siteProperties.manage_changeProperties(use_folder_tabs = useFolderTabs)

    # Remove from typesLinkToFolderContentsInFC 
    typesLinkToFolderContentsInFC = list(siteProperties.getProperty('typesLinkToFolderContentsInFC'))
    if 'SignupSheet' in typesLinkToFolderContentsInFC:
        typesLinkToFolderContentsInFC.remove('SignupSheet')
    siteProperties.manage_changeProperties(typesLinkToFolderContentsInFC = typesLinkToFolderContentsInFC)

    # Add to default_page_types
    defaultPageTypes = list(siteProperties.getProperty('default_page_types'))
    if 'SignupSheet' not in defaultPageTypes:
        defaultPageTypes.append('SignupSheet')
    siteProperties.manage_changeProperties(default_page_types = defaultPageTypes)

    # Add to parentMetaTypesNotToQuery
    #parentMetaTypesNotToQuery = list(navtreeProperties.getProperty('parentMetaTypesNotToQuery'))
    #if 'SignupSheet' not in parentMetaTypesNotToQuery:
    #    parentMetaTypesNotToQuery.append('SignupSheet')
    #navtreeProperties.manage_changeProperties(parentMetaTypesNotToQuery = parentMetaTypesNotToQuery)




    # Add a form controller that traverse to the post page so the proper state can be set
    # on the registrant.
    controller = getToolByName(self, 'portal_form_controller')
    addFormControllerAction(self, out, controller, 'validate_integrity',
                            'success', 'Registrant', None, 'traverse_to', 'string:registrant_post')

   

    # Add SignupSheet to kupu's linkable and media
    # types
    kupuTool = getToolByName(self, 'kupu_library_tool')
    linkable = list(kupuTool.getPortalTypesForResourceType('linkable'))
    mediaobject = list(kupuTool.getPortalTypesForResourceType('mediaobject'))
    if 'SignupSheet' not in linkable:
        linkable.append('SignupSheet')
    # kupu_library_tool has an idiotic interface, basically written purely to
    # work with its configuration page. :-(
    kupuTool.updateResourceTypes(({'resource_type' : 'linkable',
                                   'old_type'      : 'linkable',
                                   'portal_types'  :  linkable},
                                  {'resource_type' : 'mediaobject',
                                   'old_type'      : 'mediaobject',
                                   'portal_types'  :  mediaobject},))
    print >> out, "Added SignupSheet to kupu's linkable and mediaobject types"
    
    #install dependencies
    install_dependencies(self)
    return out.getvalue()
