"""A Registrant type that has an editable schema"""

__author__  = 'Aaron VanDerlip avanderlip@gmail.com'
__docformat__ = 'plaintext'

from AccessControl import ClassSecurityInfo

try:
  from Products.LinguaPlone.public import *
except ImportError:
  # No multilingual support
  from Products.Archetypes.public import *


#CMF
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent
from Products.CMFCore.utils import getToolByName

#ATSchemaEditorNG
from Products.ATSchemaEditorNG.ParentManagedSchema import ParentManagedSchema, ParentOrToolManagedSchema
from Products.ATSchemaEditorNG.config import ATSE_MANAGED_NONE, \
     ATSE_MANAGED_FULL

#ATContentTypes
from Products.ATContentTypes.content.document import ATDocument
from Products.ATContentTypes.content.document import finalizeATCTSchema


RegistrantSchema = BaseContent.schema.copy() + Schema(( 

StringField('first_name', widget=StringWidget(label="""first name""")),
 
StringField('last_name', widget=StringWidget(label="""last name""")) ,
 
StringField('status',
    write_permission = """SignupSheet: View Registration Info""",
    atse_managed=ATSE_MANAGED_NONE,
    read_permission="SignupSheet: View Registration Info",
    vocabulary = [('registered','registered'),('waitinglist','waitinglist')], 
    widget=SelectionWidget()
),

StringField("email", 
    read_permission = """View""",
	default_content_type = """text/plain""",
	type = """string""",
    required = 1,
    validators=('isEmail',),
    schemata = """default""",
    widget=StringWidget(
		populate = True,
		label_msgid = """label_Email""",
		macro = """widgets/string""",
		postback = True,
		cols = 30,
		label = """Email""",
		visible = {'edit': 'visible', 'view': 'visible'},
		i18n_domain = """plone""",
		maxlength = """255""",
		size = 30,
		modes = ('view', 'edit'),
		),
	),

))


RegistrantSchema['title'].required = 0
RegistrantSchema['title'].widget.visible = {'view':'invisible',
                                       'edit':'invisible'}
#Remove title and id from schema editor
RegistrantSchema['title'].atse_managed=ATSE_MANAGED_NONE
RegistrantSchema['id'].atse_managed=ATSE_MANAGED_NONE


class Registrant(ParentManagedSchema, BaseContent):
    """ Target content type to edit schema on """
    meta_type = portal_type = "Registrant"
    schema = RegistrantSchema
    global_allow = False
    
    
    def manage_afterAdd(self, item, container):
        self.updateSchemaFromEditor()
        BaseContent.manage_afterAdd(self, item, container)

    def setTitle(self, value, **kwargs):
        self.title = self.computeFullname()

    def Title(self):
        return self.computeFullname()


    def setStatus(self, value, **kwargs):
        #needs to be fixed
        if self.getStatus() == 'registered':
           pass
        else:
           self.status = self.computeStatus()

            
    def computeStatus(self):
        """to be continued """
        signupsheet = self.getParentNode()
        #this should be filtered for only the proper type
        # assume if registerd then stay that way
        event_size = signupsheet.getEventsize()
        current_size=len(signupsheet.contentIds(filter={'portal_type':'Registrant'}))
       
        if current_size <= event_size or event_size == 0:
            status = 'registered'     
        else:
            status = 'waitinglist'
            
        return status

    def computeFullname(self):
        """ compute the fullname from the firstname and lastname values """
        first_name = self.getField('first_name').getAccessor(self)()
        last_name = self.getField('last_name').getAccessor(self)()
        if first_name and last_name:
            space = ' '
        else:
            space = ''
        return "%s%s%s" % (first_name, space, last_name)
        
                
        
    def sendNotificationMail(self):
        """
        Send a confirmation message to the registrant's email.
        """

        portal_url  = getToolByName(self, 'portal_url')
        portal      = portal_url.getPortalObject()
        plone_utils = getToolByName(self, 'plone_utils')
        portal      = portal_url.getPortalObject()
        mailHost    = plone_utils.getMailHost()
        encoding    = plone_utils.getSiteEncoding()
        signupsheet = self.aq_parent
        email_response = signupsheet.getEmail_response()
        send_from_address = envelope_from = portal.getProperty('email_from_address', 'somebody@somebody.org')
        registrant_status ='pending approval'

      
        if self.status == 'waitinglist':
            registrant_status = 'waiting list'
        
        send_to_address = self.getEmail()
        
        #generate email from skin template
        message = self.signupsheet_email_response(self,
                                                  registrant_status = registrant_status,
                                                  email_repsponse = email_response)
        #generate email subject from skin
        subject = self.signupsheet_email_response_subject(self, signup_title = signupsheet.Title())
        
        mailHost.secureSend(message, send_to_address, envelope_from, subject=subject, subtype='plain', charset=encoding, debug=False, From=send_from_address)
        


registerType(Registrant)
